/********************************************************************
* boxScroll.js
*
* @desc         This script enabales you to make the site that
*               target box of side part follows window scroll.
*
* @author		Shigeru Kuratani <kuratani@benefiss.com>
* @copyright	2012, Shigeru Kuratani <Kuratani@benefiss.com>
* @license		Creative Commons License
* @version		1.0.0
* @link			http://bs.benefiss.com
* @since		File available since Release 1.0.0
********************************************************************/

/**
 * boxScroll definition
 */
function boxScroll(parentNodeId, nodeId) {
	boxScrollObj = {};
	boxScrollObj.marginTop = 0;
	this.marginTop = function(topMargin) {
					 	boxScrollObj.marginTop = topMargin;
					 };
	this.constructor = setScrollMotion(parentNodeId, nodeId);
}

/**
 * boxScroll constructor
 *
 * @param string parentNodeId  ID of parent element
 * @param string nodeId  ID of target element
 * @return void
 */
function setScrollMotion(parentNodeId, nodeId) {
	
	boxScrollObj.parentNodeId = parentNodeId;
	boxScrollObj.nodeId = nodeId;
	
	/*
	 * 1．initialize
	 * 2．initial scroll
	 * 3．set of timer
	 */
	if(navigator.userAgent.indexOf('MSIE') == -1){
		window.addEventListener('load',initialize,false);
		//window.addEventListener('load',initialScroll,false);
		window.addEventListener('load',setTimer,false);
	} else {
		window.attachEvent('onload',initialize);
		//window.attachEvent('onload',initialScroll);
		window.attachEvent('onload',setTimer);
	}
}

/**
 * setting the css 'position' of parent element and target element
 */
function initialize() {
	var parentNode = document.getElementById(boxScrollObj.parentNodeId);
	parentNode.style.position = 'relative';
	
	var node = document.getElementById(boxScrollObj.nodeId);
	node.style.position = 'absolute';
}

/**
 * initial scroll
 */
function initialScroll() {
	setScrollFunction(boxScrollObj.nodeId, boxScrollObj.marginTop);
}

/**
 * set of timer
 */
function setTimer() {
	window.setInterval('scrollMotion()', 1000);
}

/**
 * function of setting to window object
 *
 * @param void
 * @return void
 */
function scrollMotion() {
	var windowScrollTop = document.documentElement.scrollTop || document.body.scrollTop; // amount of window scroll
	var parentNodeOffsetTop = document.getElementById(boxScrollObj.parentNodeId).offsetTop; // offset of parent element
	var nodeOffsetTop = document.getElementById(boxScrollObj.nodeId).offsetTop; // offset of target element
	if (windowScrollTop > parentNodeOffsetTop) {
		if (Math.abs((windowScrollTop + boxScrollObj.marginTop) - (parentNodeOffsetTop + nodeOffsetTop)) > 1) {
			setScrollFunction(boxScrollObj.nodeId, getScrollMeasure());
		}
	} else {
		if (nodeOffsetTop > boxScrollObj.marginTop) {
			setScrollFunction(boxScrollObj.nodeId, -nodeOffsetTop);
		}
	}
}

/**
 * scroll prosess of target element
 *
 * @param string nodeId  ID of target element
 * @param number scrollAmount  amount of scroll(px)
 * @return void
 */
function setScrollFunction(nodeId, scrollAmount) {
	var nodeOffsetTop = document.getElementById(nodeId).offsetTop;
	
	var msec = 10; // interval of timer
	var amount = scrollAmount;
	
	if (amount >= 0) {
		var sc = 0;
		var off = 0;
		var m = 0;
		while (sc <= amount) {
			off += Math.round((amount - sc) / 20) + 1;
			id = window.setTimeout('setOffset(\'' + nodeId +'\',' + (off + nodeOffsetTop) + ')', m);
			sc = off;
			m += msec;
		}
	} else {
		var sc = 0;
		var off = 0;
		var m = 0;
		while (sc <= -amount) {
			off += Math.round((-amount - sc) / 20) + 1;
			id = window.setTimeout('setOffset(\'' + nodeId +'\',' + (-off + nodeOffsetTop) + ')', m);
			sc = off;
			m += msec;
		}
	}
}

/**
 * setting offset of target element
 */
function setOffset(nodeId, off) {
	document.getElementById(nodeId).style.top = off + 'px';
}

/**
 * calculate amount of scroll
 *
 * @param void
 * @return number scrollMeasure  amount of scroll
 */
function getScrollMeasure() {
	var windowScrollTop = document.documentElement.scrollTop || document.body.scrollTop; // amount of window scroll
	var parentNodeOffsetTop = document.getElementById(boxScrollObj.parentNodeId).offsetTop; // offset of parent element
	var nodeOffsetTop = document.getElementById(boxScrollObj.nodeId).offsetTop; // offset of target element
	
	if (windowScrollTop + boxScrollObj.marginTop != parentNodeOffsetTop + nodeOffsetTop) {
		var scrollMeasure = (windowScrollTop + boxScrollObj.marginTop) - (parentNodeOffsetTop + nodeOffsetTop);
	} else {
		var scrollMeasure = 0;
	}
	
	return scrollMeasure;
}
